;***************************************************************************
;
; File Name		:'M_rs232.asm"
; Title			: ATmega128 UART,  RS-232 komunikacios rutinok
; Date			:2003.02.08.	Lastmod.:[2003.08.17.]
; Version		:1.0.0
; Support telephone	:+36-70-333-4034,  Old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmaga128
;
;***************************************************************************
;	D E S C R I P T I O N
;
;	Universal Synchronous and Asynchronous serial Receiver and Transmitter (USART)
;
;Circular serial I/O buffers, with handshake handling for hardware.
;Interupt driven, thus 'transparent' to User functions
;
;You have a head and tail pointer, a constant that tells the size of the array
;
;One thing to note, is that the buffer store routine is only concerned with the
;tail pointer, and the recall routine is only concerned with the head pointer.
;Only the buffer init routine is concerned with both pointers!
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who		why
;	----	----------  ---		--------------------------------------------
;	0.01	2001.07.21  VFX		Creation (Base on 8515)
;	1.00	2003.02.08  VFX		Redesign for Atmega128
;
;***************************************************************************
;
; Init_UART
; SendChr
; SendStr
;
;
;Hardware
;***************************************************************************
;*
				;RTS pin , Adas keres DSUB9 7.
.equ	SER_HSO_PORT=PORTD	;Hardware Handshake output port
.equ	SER_HSO_DIR=DDRD	;Hardware Handshake output control
.equ	SER_HSO=7		;Hardware Handshake Output bit

				;CTS pin , Vetelkezseg nyugtazo DSUB9 8.
.equ	SER_HSI_PORT=PIND	;Hardware Handshake input port
.equ	SER_HSI_DIR=DDRD	;Hardware Handshake input control
.equ	SER_HSI=5		;Hardware Handshake Input bit


.MACRO HS_Enabled
		cbi	SER_HSO_PORT,SER_HSO		;Set low, HS to off
.endm

.MACRO HS_Disabled
		sbi	SER_HSO_PORT,SER_HSO		;Set Hi for Dis
.endm


;**************************************************************************
;* Const Def


;********************************************************************
;
;ASCII Equivalents
;
.equ	NUL=0x00	;Null
.equ	SOH=0x01	;Start of Header
.equ	STX=0x02	;Start of Text
.equ	ETX=0x03	;End of Text
.equ	EOT=0x04	;End of Transmission
.equ	ENQ=0x05	;Enquiry
.equ	ACK=0x06	;Acknowlege
.equ	BEL=0x07	;Bell
.equ	BS=0x08		;Backspace
.equ	HT=0x09		;Horizontal Tab
.equ	LF=0x0A		;Line Feed
.equ	VT=0x0B		;Vertical Tab
.equ	FF=0x0C		;Form Feed
.equ	CR=0x0D		;Carriage Return
.equ	SO=0x0E		;
.equ	SI=0x0F		;

.equ	DLE=0x10	;Delete
.equ	XON=0x11	;AKA DC1
.equ	XOFF=0x12	;AKA DC2
.equ	DC3=0x13	;
.equ	DC4=0x14	;
.equ	NAK=0x15	;Negative Acknowlege
.equ	SYN=0x16	;Sync
.equ	ETB=0x17	;
.equ	CAN=0x18	;Cancel
.equ	EM=0x19		;
.equ	SUB=0x1A	;End of File
.equ	ESC=0x1B	;
.equ	FS=0x1C		;Field Separator
.equ	GS=0x1D		;Group Separator
.equ	RS=0x1E		;Record Separator
.equ	US=0x1F		;


;From the data sheet:BAUD = XTAL/(16*(UBRR+1))
;Rearranging you get:UBRR = XTAL/(16*BAUD) - 1
;
;This would work except that any fraction will be truncated.
;For least error we want to round off the result.
;So we add 1/2 as follows: UBRR = XTAL/(16*BAUD) - 1 + 0.5
;Combining the -1 and the +0.5 we can rewrite this as follows:
;
;UBRR = INT(XTAL/(16*BAUD) - 0.5)

.equ	Rx_Buffer_Length = 24
.equ	Tx_Buffer_Length = 32

.equ	SendROMTXT = 0x02		;RS232 Tx func. Send ROM text to RS232


.equ	FlSending = 0b00000001		;Tx Send running
.equ	FlSendROM = 0b00000010		;Tx Send ROM TXT Flag


;***************************************************************************
.DSEG

Rx_Buffer:
Rx_Head:	.byte 1
Rx_Tail:	.byte 1
Rx_Data:	.byte Rx_Buffer_Length

Tx_Buffer:
Tx_Head:	.byte 1
Tx_Tail:	.byte 1
Tx_Data:	.byte Tx_Buffer_Length


UARTFlg:	.byte 1			;UART Flags
ROMTXT:		.byte 3			;ROM TXT kuldeskor az aktualis ROM pozicio cime



;*************************************************************************
;*  Text Const
;*
.CSEG


;************************************************************************
;** Init_UART1
;** Set up the inital states for the buffers, and make sure the handshake
;** is turned on (Not the Shaddap state), so we can talk.
;**
;**
;************************************************************************
Init_UART1:	ldi	R16,0
		sts	Tx_Tail,R16
		sts	Tx_Head,R16
		sts	Rx_Tail,R16
		sts	Rx_Head,R16
		sts	UARTFlg,R16

		sts	CMD_Len,R16
		sts	CMD_Pos,R16

						;Set up Hardware HS I/O bits
		sbi	SER_HSO_DIR,SER_HSO	;Set as output
		HS_Enabled

		cbi	SER_HSI_PORT,SER_HSI	;Set as input, Pull-up
		cbi	SER_HSI_DIR,SER_HSI

		ldi	R16,High(SYSCLK/(16*BaudSpeed1)-1)
		sts	UBRR1H,R16
		ldi	R16,Low(SYSCLK/(16*BaudSpeed1)-1)
		sts	UBRR1L,R16		;BaudRate Gen.

		ldi	R16,0b00000000		;RXCn, TXCn, UDREn, FEn, DORn, UPEn, U2Xn, MPCMn
		sts	UCSR1A,R16

		ldi	R16,0b11011000		;RXCIE,TXCIE,UDRIE,RXEN,TXEN,UCSZn2,RXB8,TXB8
		sts	UCSR1B,R16

		ldi	R16,0b00000110		;, UMSELn, UPMn1, UPMn0, USBSn, UCSZn1, UCSZn0, UCPOLn
		sts	UCSR1C,R16
		ret

ECopyr:	  	.DB "(C) 2003 VFX ATmega128 Service Terminal [2003.02.08.]",CR
	  	.DB "Firmware version ",MAJOR_REV,".",MINOR_REV,MINOR_REVB," "
	  	.DB "Latest Upd.: 2003.08.17.",CR,0


;***********************************************
;* Megnezi, hogy kell-e ujrainditani a Tx muveletet

TxRunning:	lds	ZL,Tx_Head
		lds	ZH,Tx_Tail
		cp	ZL,ZH			;ha azonosak akkor nincs mit tenni
		breq	TxRun1
		lds	ZL,UARTFlg
		andi	ZL,FlSending		;Tx folyamatban?
		brne	TxRun1			;ha igen nincs mit tenni
		rjmp	USART1_TXC
TxRun1:		ret



;***************************************************************************
;***************************************************************************
;***************************************************************************
;**   UART TX Functions
;***************************************************************************
; kod			  Bufferbe 		    Osszesen
; 0x02 - send ROM TXT ->  0xE0, 0x02, low addr, hi addr ; 4 byte
; 0xE0 - send 0xE0    ->  0xE0, 0xE0 			; 2 byte


;************************************************************************
;** Calc Tx Buffer Free Space
; In:	-
; Out:	R18 - Tx_Tail
;	R19 - Tx_Head
;	R20 - Free Space in Tx Buffer
; Alt:  R21
;
; Tail < Head -> Free= LEN+Tail-Head-1
; Tail = Head -> Free= LEN
; Tail > Head -> Free= Tail-Head-1
;
FreeInTxBuff:	lds	R19,Tx_Head
		lds	R18,Tx_Tail
		ldi	R20,Tx_Buffer_Length
		mov	R21,R18
		sub	R21,R19
		breq	FreeInTx1		;T=H, Free= Tx_Buffer_Length
		brcs	FreeInTx2
                clr	R20
FreeInTx2:	add	R20,R21
		dec	R20
FreeInTx1:	ret

;************************************************
;* The RS232 Tx FIFO manager ROM TEXT		*
;* In: (R16, R17) address word of text (L,H)		*
;* Out: c=1 hiba				*
;************************************************
SendStr:	rcall	FreeInTxBuff		;R18,R19-t is beallitja
		cpi	R20,4
		brcs	ExitwError		;ha <4 nincs eleg hely! Hiba!

		clr	R18			;R18=0
		ldi	ZL,low(Tx_Data)		;Z to the addr of txbuffer (FIFO)
		ldi	ZH,high(Tx_Data)
		add	ZL,R19
		adc	ZH,R18			;add address offset...  z = @Buf[Tx_head]
		push	R16
		ldi	R16,0xE0		;Alternative function indicator

		cli

		rcall	StoreChar
		ldi	R16,SendROMTXT		;Buf[Tx_head]=SendROMTXT indicate Send ROM TXT
		rcall	StoreChar
		pop	R16
		rcall	StoreChar		;Buf[Tx_head]=RomTXT low address

		sei

		mov	R16,R17			;Store RomTXT high address
		rcall	StoreChar
		sts	Tx_Head,R19		;Store new pointer

		rcall	TxRunning

		clc				;Nincs hiba
		ret

ExitwError:	sec				;Return with error
		ret


SendStrW:	rcall	SendStr
		brcs	SendStrW
		ret



;************************************************
;* The RS232 Tx FIFO manager Send CHAR		*
;* in: R16 char 				*
;* out c=1 hiba					*
;************************************************
SendChr:	rcall	FreeInTxBuff		;R18,R19-t is beallitja
		cpi	R20,2
		brcs	ExitwError		;ha <1 nincs elg hely! Hiba!

		clr	R18			;R18=0
		ldi	ZL,low(Tx_Data)		;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx_Data)
		add	ZL,R19
		adc	ZH,R18			;add address offset...  z = @Buf[txhead]

		cpi	R16,0xE0		;Alternative kod?
		brne	only1char		;ha igen akkor ketszer kell atkuldeni!!
		rcall	StoreChar
only1char:	rcall	StoreChar
		sts	Tx_Head,R19

		rcall	TxRunning

		clc				; Nincs hiba
		ret

SendChrW:	push	ZL
		push	ZH
		rcall	SendChr
		pop	ZH
		pop	ZL
		brcs	SendChrW
		ret


;***********************************************
; Subrutin for SendROMTXT & SendChar

StoreChar:	st	Z+,R16			;Store char
		inc	R19
		cpi	R19,Tx_Buffer_Length
		brcs	StoreC1			;TxHead<Tx_Buffer_Length akkor jo
		clr	R19			;TxHead=0
		ldi	ZL,low(Tx_Data)		;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx_Data)
StoreC1:	ret


;***********************************************
; UART Tx Interrupt rutinok


;**********************************************
USART1_DRE:	reti


;**********************************************
;* UART TX COMPLET
;**********************************************
USART1_TXC:	push	ZL
		push	ZH
		in	ZL,SREG			;preserve main OS status reg.
		push	ZL
		push	R16
		push	R0
		push	R17
		in	ZL,RAMPZ
		push	ZL

		lds	R16,UCSR1A
		ori	R16,(1<<TxC1)		;Clear Tx Complete Flag
		sts	UCSR1A,R16

		lds	ZL,UARTFlg
		andi	ZL,FlSendROM
                breq	UTxdre0
		rjmp	SendTXT			;ha aktiv ROM tartalom kuldes van ugras


UTxdre0:	lds	R17,Tx_Tail
		lds	ZL,Tx_Head
		cp	R17,ZL
		brne	Vanmeg			;ha nem egyenloek, akkor van meg kuldendo

		clr	R16			;Flagek torlese, alapallapot
		STS	UARTFlg,R16
		rjmp	NoMore			;itt a vege nincs tobb adat

Vanmeg: 	clr	R0
		ldi	ZL,low(Tx_Data)		;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx_Data)
		add	ZL,R17
		adc	ZH,R0 			;z = @Buf[Tx_Tail]
		ld	R16,Z+			;R16=Buf[txtail]
		inc	R17			;Tail+1
		cpi	R17,Tx_Buffer_Length	;buffer vegere ertunk?
                brcs	txn1			;meg nem akkor ugras

		ldi	ZL,low(Tx_Data)		;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx_Data)
		clr	R17

txn1:		cpi	R16,0xE0		;Alternative Send Command?
		breq	NormalCh

		rjmp	schar			;ha nem akkor send char

						;itt lehet 'send ROM txt' or 'send 0xE0'
NormalCh:
		ld	R16,Z+			;R16=Buf[txtail]
		inc	R17			;Tail+1
		cpi	R17,Tx_Buffer_Length	;buffer vegere ertunk?
                brcs	txn1x			;meg nem akkor ugras
		ldi	ZL,low(Tx_Data)		;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx_Data)
		clr	R17

txn1x:		cpi	R16,SendROMTXT		;Send ROMTXT Command?
		brne	schar

;Send ROM TEXT
		ld	R16,Z+			;R16=Low address of txt
		inc	R17			;Tail+1
		cpi	R17,Tx_Buffer_Length	;vegere ertunk?
                brcs	txn2			;meg nem akkor ugras
		ldi	ZL,low(Tx_Data)		;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx_Data)
		clr	R17

txn2:		ld	R0,Z+			;R0=High address of txt
		inc	R17			;Tail+1
		cpi	R17,Tx_Buffer_Length	;vegere ertunk?
                brcs	txn3			;meg nem akkor ugras
		ldi	ZL,low(Tx_Data)		;Pointer to the txbuffer (FIFO)
		ldi	ZH,high(Tx_Data)
		clr	R17

txn3:		lsl	R16			;Addrss * 2 word to byte conv.
		STS	ROMTXT+0,R16
		rol	R0
		STS	ROMTXT+1,R0
		ldi	R16,0
		rol	R16
		STS	ROMTXT+2,R16		;Save ROM Addr
		sts	Tx_Tail,R17		;Save TxTail

		lds	R16,UARTFlg
		ori	R16,FlSendROM		;Sending ROMTXT flag
		sts	UARTFlg,R16

SendTXT:					;ROM TXT sending...
		LDS	ZL,ROMTXT+0
		LDS	ZH,ROMTXT+1
		lds	R0,ROMTXT+2
		out	RAMPZ,R0
                ELPM	R0,Z+			;R0 = [RAMPZ:Z]
		STS	ROMTXT+0,ZL
		STS	ROMTXT+1,ZH
		in	R16,RAMPZ
		sts	ROMTXT+2,R16

		mov	R16,R0
		CPI	R16,0
		BRNE	MoreTXT			;VAN MEG SZOVEG

		lds	ZL,UARTFlg
		cbr	ZL,FlSendROM		;nincs tovabb ROM send
		sts	UARTFlg,ZL
		rjmp	UTxdre0


;single char send
schar:		sts	Tx_Tail,R17		;Save TxTail

MoreTXT:	sts	UDR1,R16
		lds	R16,UARTFlg
		ori	R16,FlSending		;ujra atvitel van
		STS	UARTFlg,R16

NoMore:
		pop	ZL
		out	RAMPZ,ZL
		pop	R17
		POP	R0
		POP	R16
		POP	ZL
		OUT	SREG,ZL
		POP	ZH
		POP	ZL
		RETI





;***************************************************************************
;***************************************************************************
;***************************************************************************
;**   UART RX Functions
;***************************************************************************


;*******************************************
;* UART RX COMPLET
;*******************************************
USART1_RXC:	push	ZL
		push	ZH
		in	ZL,SREG			; preserve main OS status reg.
		push	ZL
		push	R16
		push	R2

		clr	R2
		ldi	ZL,low(Rx_Data)		; ptr to our rxbuffer... z = Buf[rxtail]
		ldi	ZH,high(Rx_Data)
		lds	R16,Rx_Head
		add	ZL,R16
		adc	ZH,R2

						;itt ki kellene olvasni a statust, a rafutas, frame error stb miatt

		lds	R2,UDR1			; get the incomming char
		st	Z,R2			; Buf[head] = UDR
		inc	R16
		cpi	R16,Rx_Buffer_Length
		brlo	Rs232inx
		clr	R16			; Circle around to the first buffer pos.
Rs232inx:	sts	Rx_Head,R16
        					;Kiszamolja van-e meg hely a bufferben

		ldi	ZL,Rx_Buffer_Length
		lds	R2,Rx_Tail
		sub	R2,R16
		breq	RStxPSC4			;T=H, akkor Kb_ScData_Lenght hely van
		brcs	RStxPSC2
		clr	ZL
RStxPSC2:
		add	ZL,R2
		dec	ZL
RStxPSC4:	cpi	ZL,1			;legalabb 1 hely maradjon ures!
		brsh	RS232IntExit

                HS_Disabled			;ha nincs hely tiltjuk a komunikaciot CTS/RTS vonalon

RS232IntExit:	pop	R2
		pop	R16
		pop	ZL
		out	SREG,ZL			; restore previous status reg
		pop	ZH
		pop	ZL
		reti				; return to normal pgm status.


;************************************************
;* The RS232 RxD FIFO manager			*
;* In: -                                        *
;* Out C=1, akkor R0=RS232 rx-ed char		*
;************************************************
Rs232GetByte:	clr	R0
		lds	R18,Rx_Tail
		lds	R19,Rx_Head
		cp	R19,R18
		breq	Rs232GetBytexit
		ldi	ZL,low(Rx_Data)		; Pointer to the rxbuffer (FIFO)
		ldi	ZH,high(Rx_Data)
		add	ZL,R18
		adc	zh,R0			; add address offset...  z = &Buf[Tail]
		ld	R0,Z			; R0 = Buf[Tail++]
		inc	R18
		cpi	R18,Rx_Buffer_Length	; test for the end of the data buffer
		brlo	Rs232Gbx
		clr	R18			; Circle around to the first buffer pos.
Rs232Gbx:	sts	Rx_Tail,R18

		sec				;Valid Char
		ret
Rs232GetBytexit:
		HS_Enabled			;CTS/RTS atvitel engedelyezett

		clc				; Return 'No' bytes in yet.
		ret

;****************************************
;* And The RS232 GetChar Loop		*
;* wait until a char comes in 		*
;****************************************
GetChar:	rcall	Rs232GetByte
		brcc	GetChar
		ret


