;***************************************************************************
; 
; File Name		:'Main.asm"
; Title			:Matmatikai rutinok, test verzio 
; Date			:2003.07.16.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmega128
; 
;***************************************************************************
;	D E S C R I P T I O N
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date      who 	why
;	----	----------  ---		-----------------------------------
;	0.01	2003.07.16. VFX		Creation
;
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	f=external 16MHz (t=-.--ns)
;*
;***************************************************************************
;* Const Def


;**************************************************************************
;* Hardware Def.
;

;***************************************************************************
;** VARIABLES
.DSEG

;***************************************************************************
.ESEG

;***************************************************************************
.CSEG


; -----------------------------
; Prepare to multiply or divide
; -----------------------------
; This routine is called in succession from multiply and divide to prepare
; two mantissas by setting the leftmost bit that is used for the sign.
;  m1: R25, R24:R21
;  m2:  R6, R5:R2
;
;  T flag = Sign of result
;  R6,R25 corrected exponent
;  R5,R24 restored sign bit of mantissa
;
;   cycles +  (RET) =  Cycles

PREP_Mul_Div:
		mov		R16,R5
		eor		R16,R24
		bst		R16,7		;T flag = result sign bit

		ldi		R16,0xA0	;Binary point offset is 0x20 to integer number
		ldi		R17,0x80	;MSB sign bit

		tst		R25
		brne	PREP_MD01	;First number is floating point format
		add		R25,R16
		rjmp	PREP_MD02

PREP_MD01:
		or		R24,R17		;restore msb bit
PREP_MD02:		
		tst		R6
		brne	PREP_MD03	;Second number is floating point format
		add		R6,R16
		rjmp	PREP_MD04

PREP_MD03:
		or		R5,R17		;restore msb bit
PREP_MD04:		
		sub		R25,R17
		sub		R6,R17		;shift zero point by 0x80 (back to 2's complement)
		ret

; ------------------------------
; THE 'MULTIPLICATION' OPERATION
; ------------------------------
; (offset: $04 'multiply')
;
;  m1: R25, R24:R21
;  m2:  R6, R5:R2
;  -------------------
; Res:  R6, R5:R2
;
;   cycles +  (RET) =  Cycles
;
multiply:
		rcall	PREP_Mul_Div
		rcall	mult32

		ldi		R16,64		;max 64 bitet kell vegigrotalni
Mult0:	tst		R15
		brne	Mult1
		mov		R15,R14
		mov		R14,R13
		mov		R13,R12
		mov		R12,R11
		mov		R11,R10
		mov		R10,R9
		mov		R9,R8
		clr		R8
		subi	R16,8
		cpi		R16,8
		brne	Mult0
Mult1:
								;legnagyobb helyiertek felrotalasa bit7-ig
		sbrc	R15,7
		 rjmp	Mult2
		lsl		R8
		rol		R9
		rol		R10
		rol		R11
		rol		R12
		rol		R13
		rol		R14
		rol		R15
		dec		R16
		brne	Mult1
Mult2:							;R16 ban a rotalas eredmenye
		mov		R2,R12
		mov		R3,R13
		mov		R4,R14
		mov		R5,R15
		ldi		R17,0xFF
		bld		R17,7			;get result sign bit
		and		R5,R17			;set result sign bit

		ldi		R17,64
		sub		R17,R16

		add		R6,R25			;2^x *2^y = 2^(x+y)
		brvs	NumerTooBig
		sub		R6,R17			;shift exponent by normalized value
		brcs	NumerTooBig
		ldi		R16,0x80
		sub		R6,R16			;set offset by 0x80
		clc
		ret
NumerTooBig:
		sec
		ret



;***************************************************
;* Mutiply 32x32 -> 64 bit
;* 
;*  R24:R21 x R5:R2 -> R15:R8
;*
;*  86 cycles + 4 (RET) = 90 Cycles
;*
mult32:
		clr	R16
		mul	R21,R2
		movw	R8,R0
		clr	R10
		clr	R11
		movw	R12,R10
		movw	R14,R10
		mul	R22,R2
		add	R9,R0
		adc	R10,R1
		mul	R23,R2
		add	R10,R0
		adc	R11,R1
		mul	R24,R2
		add	R11,R0
		adc	R12,R1

		mul	R21,R3
		add	R9,R0
		adc	R10,R1
		adc	R11,R16
		adc	R12,R16
		adc	R13,R16
		mul	R22,R3
		add	R10,R0
		adc	R11,R1
		adc	R12,R16
		adc	R13,R16
		mul	R23,R3
		add	R11,R0
		adc	R12,R1
		adc	R13,R16
		mul	R24,R3
		add	R12,R0
		adc	R13,R1

		mul	R21,R4
		add	R10,R0
		adc	R11,R1
		adc	R12,R16
		adc	R13,R16
		adc	R14,R16
		mul	R22,R4
		add	R11,R0
		adc	R12,R1
		adc	R13,R16
		adc	R14,R16
		mul	R23,R4
		add	R12,R0
		adc	R13,R1
		adc	R14,R16
		mul	R24,R4
		add	R13,R0
		adc	R14,R1

		mul	R21,R5
		add	R11,R0
		adc	R12,R1
		adc	R13,R16
		adc	R14,R16
		adc	R15,R16
		mul	R22,R5
		add	R12,R0
		adc	R13,R1
		adc	R14,R16
		adc	R15,R16
		mul	R23,R5
		add	R13,R0
		adc	R14,R1
		adc	R15,R16
		mul	R24,R5
		add	R14,R0
		adc	R15,R1
		ret

