;***************************************************************************
;
; File Name		:'bmp.asm"
; Title			:Windows Bitmap utility (*.bmp) 
; Date			:2003.06.23.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmega128
;
;***************************************************************************
;	D E S C R I P T I O N
;
;Basic Description 
;
;This file format is the MS-Windows standard format.
;It holds black&white-, 16-color, 256-color and Truecolor images.
;The palletized 16-color and 256-color images may be compressed via run
;length encoding. Notice there is also a OS/2-BMP format.
;
;Basic File Format 
;
;Name		Size		Description
;Header		14 bytes	Windows Structure: BITMAPFILEHEADER
; Signature	2 bytes		'BM'
; FileSize	4 bytes		File size in bytes
; reserved	4 bytes		unused (=0)
; DataOffset	4 bytes		File offset to Raster Data
;InfoHeader	40 bytes	Windows Structure: BITMAPINFOHEADER
; Size		4 bytes		Size of InfoHeader =40 
; Width		4 bytes		Bitmap Width
; Height	4 bytes		Bitmap Height
; Planes	2 bytes		Number of Planes (=1)
; BitCount	2 bytes		Bits per Pixel
;				 1 = monochrome palette. NumColors = 1  
;				 4 = 4bit palletized. NumColors = 16  
;				 8 = 8bit palletized. NumColors = 256 
;				 16 = 16bit RGB. NumColors = 65536 (?) 
;				 24 = 24bit RGB. NumColors = 16M
; Compression	4 bytes		Type of Compression
;				 0 = BI_RGB  no compression
;				 1 = BI_RLE8 8bit RLE encoding
;				 2 = BI_RLE4 4bit RLE encoding
; ImageSize	4 bytes		(compressed) Size of Image
;				It is valid to set this =0 if Compression = 0
; XpixelsPerM	4 bytes		horizontal resolution: Pixels/meter
; YpixelsPerM	4 bytes		vertical resolution: Pixels/meter
; ColorsUsed	4 bytes		Number of actually used colors
; ColorsImportant 4 bytes	Number of important colors
;				 0 = all
;ColorTable			4 * NumColors bytes
;				present only if Info.BitsPerPixel <= 8
;				colors should be ordered by importance
; Red		1 byte		Red intensity
; Green		1 byte		Green intensity
; Blue		1 byte		Blue intensity
; reserved	1 byte		unused (=0)
;  ...				repeated NumColors times
;
;Raster Data	Info.ImageSize bytes The pixel data
;
;
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		--------------------------------------------
;	0.01	2003.06.23  VFX		Creation
;
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	Oscillator: f=external, 16.0000 MHz (T=62.5ns)
;*
;***************************************************************************
;* Const Def

;Header
.equ	BMPSignature	= 0		;offset address
.equ	BMPFileSize	= 2
.equ	BMPreserved1	= 6
.equ	BMPDataOffset	= 10

;InfoHeader
.equ	BMPSize		= 14
.equ	BMPWidth	= 18
.equ	BMPHeight	= 22
.equ	BMPPlanes	= 26
.equ	BMPBitCount	= 28
.equ	BMPCompression	= 30
.equ	BMPImageSize	= 34
.equ	BMPXpixelsPerM	= 38
.equ	BMPYpixelsPerM	= 42
.equ	BMPColorsUsed	= 46
.equ	BMPColorsImportant=50
.equ	BMPColorTable	= 54


;**************************************************************************
;* Hardware Def.
;


;***************************************************************************
;**** VARIABLES
.DSEG
;***************************************************************************
.ESEG

;***************************************************************************
;****		C O D E 
;****
;***************************************************************************
.CSEG

PutImg:
		ldi	ZL,Low(audi*2+BMPColorTable)
		ldi	ZH,high(audi*2+BMPColorTable)
		ldi	R16,(audi+BMPColorTable/2)/32768
		out	RAMPZ,R16
		ldi	R17,16
		call	WriteLUT1

		ldi	R20,SRAMBASE		;SRAM RAM Base Address for DMA
		sts	ADR_EXLH,R20		;Lower memory Banks
		inc	R20
		sts	ADR_EXHL,R20		;Continous 48kb VideoMemory
		inc	R20
		sts	ADR_EXHH,R20
		clr	R2

		ldi	XH,S1D_DISPLAY_HEIGHT

		ldi	YL,Low(54624)
		ldi	YH,High(54624)
		ldi	R19,S1D_DISPLAY_SCANLINE_BYTES

FillVideox:
		push	YL
		push	YH
		ldi	R18,S1D_DISPLAY_SCANLINE_BYTES
                        
FV1:
		elpm	R0,Z+
		st	Y+,R0
		dec	R18
		brne	FV1

		pop	YH
		pop	YL
		sub	YL,R19
		sbc	YH,R2
		dec	XH
		brne	FillVideox
		ret



