;***************************************************************************
;
; File Name		:'gps.asm"
; Title			:GPS-PS1E GPS modul Driver
; Date			:2003.06.26.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmega128
;
;***************************************************************************
;	D E S C R I P T I O N
;
;This file comunicate with GPS-PS1E GPS modul by SiRF binary protocoll
;
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		--------------------------------------------
;	0.01	2003.06.26  VFX		Creation
;
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	Oscillator: f=external, 16.0000 MHz (T=62.5ns)
;*
;***************************************************************************
;* Const Def

.EQU	INTGPS_DIR = DDRE
.EQU	INTGPS_PORT= PORTE
.EQU	INTGPS	  = 6

.equ	GPSRx_Buffer_Length = 254
.equ	GPS_MessageBufferLen = 1032


.equ	MeasureNavigation = 0x02	;Measure Navigation Data Out - Msg ID. 2


;**************************************************************************
;* Hardware Def.
;

.equ	GPSIRQ	= 0		;0. bit =1 GPS UART Data available


;***************************************************************************
;**** VARIABLES
.DSEG

GPSRx_Buffer:
GPSRx_Head:	.byte 1
GPSRx_Tail:	.byte 1
GPSRx_Data:	.byte GPSRx_Buffer_Length


GPS_Message:	.byte GPS_MessageBufferLen
GPS_MsgLen:	.byte 2

GPS_Seq:	.byte 1				;GPS state machine


GPSXCoord:	.byte 4 			;egymas utan kell alljanak!!
GPSYCoord:	.byte 4
GPSZCoord:	.byte 4

GPSFlag:	.byte 1


;***************************************************************************
.ESEG

;***************************************************************************
;****		G P S  I N T.   &   C O D E 
;****
;***************************************************************************
.CSEG

GPS_Init:	
		clr	R16
		sts	GPSRx_Tail,R16
		sts	GPSRx_Head,R16
		sts	GPSFlag,R16
		rcall	ClearMsgBuff


		cbi	INTGPS_DIR,INTGPS
		sbi	INTGPS_PORT,INTGPS	;PE6 Input Pull-up

		in	R16,EICRB		;INT6 lesz a GPS Rx Interrupt
		andi	R16,0b11001111
		out	EICRB,R16		;INT6 actice to Low level 

		ldi	R16,64
		out	EIFR,R16		;Pending Int6 cleared

		in	R16,EIMSK
		ori	R16,64			;INT6 enabled
		out	EIMSK,R16

		ret

GPS_Syncronize:

		ldi	R16,Low(GPSS)
		ldi	R17,High(GPSS)
		call	SendStrW

		ldi	R16,Low(5*SYSTACK)
		sts	SCNT1+0,R16
		ldi	R16,High(5*SYSTACK)
		sts	SCNT1+1,R16		;varakozas max 5sec
		clr	R20
		clr	R21			;0xB0, 0xB3 kereseshez
		clr	R23

GPS_Sync0:
		sleep
		nop

		lds	R16,SCNT1+0
		andi	R16,16
		mov	R17,R23
		mov	R23,R16
		eor	R16,R17
		breq	GPS_Sync00
		call	TogleGreenLED
GPS_Sync00:		

		rcall	GetGPSByte
		brcs	GPS_Sync1

		lds	R16,SCNT1+0
		lds	R17,SCNT1+1
		or	R17,R16
		brne	GPS_Sync0


GPSNotFound:
		ldi	R16,Low(GPSNF)
		ldi	R17,High(GPSNF)
		rjmp	GPS_SyncEnd

GPS_Sync1:	
		mov	R21,R20
		mov	R20,R0
		cpi	R21,0xB0
		brne	GPS_Sync0
		cpi	R20,0xB3
		brne	GPS_Sync0
						;GPSFound
		ldi	R16,Low(GPSF)
		ldi	R17,High(GPSF)
GPS_SyncEnd:
		call	SendStrW
		sbi	LED1_PORT,LED1		;Green LED Off
		ret


GPSF:		.db	"found.",CR,0
GPSNF:		.db	"not found.",CR,0
GPSS:		.db	"Searching for GPS... ",0

ClearMsgBuff:	clr	R16
		sts	GPS_MsgLen+0,R16
		sts	GPS_MsgLen+1,R16
		sts	GPS_Seq,R16
		ret



;*******************************************
;* GPS UART RX COMPLET
;*******************************************
EXT_INT6:	push	ZL
		push	ZH
		in	ZL,SREG			; preserve main OS status reg.
		push	ZL
		push	R16
		push	R2

		clr	R2
		ldi	ZL,low(GPSRx_Data)	; ptr to our rxbuffer... z = Buf[rxtail]
		ldi	ZH,high(GPSRx_Data)
		lds	R16,GPSRx_Head
		add	ZL,R16
		adc	ZH,R2


		lds	R2,ADR_GPSD		; get the incomming char
		st	Z,R2			; Buf[head] = UDR
		inc	R16
		cpi	R16,GPSRx_Buffer_Length
		brlo	GPSinx
		clr	R16			; Circle around to the first buffer pos.
GPSinx:	sts	GPSRx_Head,R16
        		
						;Itt kell majd megnezni van-e meg hely!!


GPSintExit:	pop	R2
		pop	R16
		pop	ZL
		out	SREG,ZL			; restore previous status reg
		pop	ZH
		pop	ZL
		reti				; return to normal pgm status.


;************************************************
;* The GPS RxD FIFO manager			*
;* In: -                                        *
;* Out C=1, akkor R0=RS232 rx-ed char		*
;************************************************
GetGPSByte:	clr	R0
		lds	R18,GPSRx_Tail
		lds	R19,GPSRx_Head
		cp	R19,R18
		breq	GetGPSBytExit
		ldi	ZL,low(GPSRx_Data)		; Pointer to the rxbuffer (FIFO)
		ldi	ZH,high(GPSRx_Data)
		add	ZL,R18
		adc	zH,R0			; add address offset...  z = &Buf[Tail]
		ld	R0,Z			; R0 = Buf[Tail++]
		inc	R18
		cpi	R18,GPSRx_Buffer_Length	; test for the end of the data buffer
		brlo	GPSGbx
		clr	R18			; Circle around to the first buffer pos.
GPSGbx:		sts	GPSRx_Tail,R18

		sec				;Valid Char
		ret
GetGPSBytExit:
		clc				; Return 'No' bytes in yet.
		ret



;***************************************************
;*
GPSTerminal:
		rcall	GetGPSByte
		brcs	GPSTerm1
		ret

GPSTerm1:				;R0-ban a vett adat
		mov	R17,R0
		lds	R16,GPS_Seq
		cpi	R16,0
		breq	SHead1
		cpi	R16,1
		breq	SHead2
		cpi	R16,2
		breq	SPayLL1
		cpi	R16,3
		breq	SPayLL2
		cpi	R16,4
		breq	FillPayLoad
		cpi	R16,5
		breq	MsgCheckSH
		cpi	R16,6
		breq	MsgCheckSL
		cpi	R16,7
		breq	MsgBrkH
		cpi	R16,8
		breq	MsgBrkL

		rjmp	GPSTerminal

GPSTerm2:	
		sts	GPS_Seq,R16
		rjmp	GPSTerminal


;***********************************		
SHead1:		cpi	R17,0xA0
		brne	GPSTerminal
					;Header 0xA0 megjott keressuk a 0xA2-t
		inc	R16
		rjmp	GPSTerm2

;***********************************
SHead2:
		cpi	R17,0xA2
		breq	SHead20
		dec	R16
		rjmp	GPSTerm2
SHead20:				;Header 0xA2 megjott, Lenght varjuk 2 byte-on 
		inc	R16
		rjmp	GPSTerm2
		

;***********************************
SPayLL1:
		rcall	StoreGPSData
		inc	R16
		rjmp	GPSTerm2

;***********************************
SPayLL2:
		rcall	StoreGPSData
                			;ide kell, meg PayLoadLenght>1024 hiba
		inc	R16
		rjmp	GPSTerm2

;***********************************
FillPayLoad:
		rcall	StoreGPSData
		sbiw	XL,2		;PayLoadLenght kimarad a hosszbol
		cp	XL,R1
		cpc	XH,R2
		brne	GPSTerm2
		inc	R16
		rjmp	GPSTerm2

;***********************************
MsgCheckSH:
		rcall	StoreGPSData
		inc	R16
		rjmp	GPSTerm2

;***********************************
MsgCheckSL:
		rcall	StoreGPSData
		inc	R16
		rjmp	GPSTerm2

;***********************************
MsgBrkH:	cpi	R17,0xB0
		breq	MsgBrkH1
		rcall	ClearMsgBuff
		rjmp	GPSTerm2
MsgBrkH1:	inc	R16
		rjmp	GPSTerm2


;***********************************
MsgBrkL:	cpi	R17,0xB3
		breq	MsgBrkL1
		rcall	ClearMsgBuff
		rjmp	GPSTerm2
MsgBrkL1:	rcall	Check_SiRF_Msg
		rcall	ClearMsgBuff
		rjmp	GPSTerminal



;***********************************
;***********************************
StoreGPSData:
		ldi	YL,Low(GPS_Message)
		ldi	YH,High(GPS_Message)
		ldd	R2,Y+0				;PayLoad Lenght High
		ldd	R1,Y+1				;PayLoad Lenght Low
		lds	XL,GPS_MsgLen+0
		lds	XH,GPS_MsgLen+1
		add	YL,XL
		add	YH,XH
		st	Y,R17
		adiw	XL,1
		sts	GPS_MsgLen+0,XL
		sts	GPS_MsgLen+1,XH			;Y current lenght
		ret
	


;***********************************
;* Feldologzza a Binary SiRF MSG sort
; 15 bit PayLoadLenght
; <1023 byte PayLoad
; 15 bit CheckSum
;
Check_SiRF_Msg:
		ldi	XL,low(GPS_Message+2)
		ldi	XH,high(GPS_Message+2)
		ld	R16,X+
		cpi	R16,MeasureNavigation
		breq	Measured
	
		ret


Measured:
		ldi	R16,12			;copy X,Y,Z pos
		ldi	YL,low(GPSXCoord)
		ldi	YH,high(GPSXCoord)
copy1:		ld	R0,X+
		st	Y+,R0
		dec	R16
		brne	copy1
		

		lds	R19,GPSXCoord+0
		lds	R18,GPSXCoord+1
		lds	R17,GPSXCoord+2
		lds	R16,GPSXCoord+3
		call	SLTOA
		ldi	ZL,low(ASCIIBuf)
		ldi	ZH,high(ASCIIBuf)
		ldi	YL,low(S1D_DISPLAY_SCANLINE_BYTES*200+30)
		ldi	YH,high(S1D_DISPLAY_SCANLINE_BYTES*200+30)
		call	PutStrSRAM


		lds	R19,GPSXCoord+4
		lds	R18,GPSXCoord+5
		lds	R17,GPSXCoord+6
		lds	R16,GPSXCoord+7
		call	SLTOA
		ldi	ZL,low(ASCIIBuf)
		ldi	ZH,high(ASCIIBuf)
		ldi	YL,low(S1D_DISPLAY_SCANLINE_BYTES*212+30)
		ldi	YH,high(S1D_DISPLAY_SCANLINE_BYTES*212+30)
		call	PutStrSRAM

		lds	R19,GPSXCoord+8
		lds	R18,GPSXCoord+9
		lds	R17,GPSXCoord+10
		lds	R16,GPSXCoord+11
		call	SLTOA
		ldi	ZL,low(ASCIIBuf)
		ldi	ZH,high(ASCIIBuf)
		ldi	YL,low(S1D_DISPLAY_SCANLINE_BYTES*224+30)
		ldi	YH,high(S1D_DISPLAY_SCANLINE_BYTES*224+30)
		call	PutStrSRAM

		call	StartDMA

		lds	R16,GPSFlag
		cpi	R16,0
		brne	GPSDebugInfoKi
               	ret

GPSDebugInfoKi:

		ldi	XL,low(GPS_Message)
		ldi	XH,high(GPS_Message)
		ld	YH,X+
		ld	YL,X+
		
SendU:		ld	R0,X+
		push	YL
		push	YH
		push	XL
		push	XH
		call	HexToUART
		ldi	R16,' '
		call	SendChrW
		pop	XH
		pop	XL
		pop	YH
		pop	YL
		sbiw	YL,1
		brne	SendU
		ldi	R16,CR
		call	SendChrW

		ret
