;***************************************************************************
;
; File Name		:'memory.asm"
; Title			:Memory Manager for VFX's DataLog
; Date			:2003.08.03.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034,  old: +36-30-9541-658 VFX
; Support fax		:
; Support Email		:info@vfx.hu
; Target MCU		:ATmega128
;
;***************************************************************************
;	D E S C R I P T I O N
;
;
; MCB
; 0. byte - Type/Flag
;		7. =1 last block
;		6. =1 first block
;		5. =1 used block
;		4. = nc
;		3:0 = process number Hi part
; 1. byte - process number low part
; 2. word - low part of length
; 4. byte - hi part of length
; 5. word - low part offset of prev. MCB  
; 7. byte - hi part offset of prev. MCB
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y
;
;
;       rev.      date      who  	why
;	----	----------  ---		------------------------------------
;	0.01	2003.08.03  VFX		Creation
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	SYSCLK: f=16.000 MHz (T= 62.5 ns)
;*
;***************************************************************************

.equ	MCB_Type  = 0
.equ	MCB_Proc  = 1
.equ	MCB_LenL  = 2
.equ	MCB_LenH  = 4
.equ	MCB_PrevL = 5
.equ	MCB_PrevH = 7
.equ	MCB_Size  = 8


.equ	MCBLast  = 7
.equ	MCBFirst = 6
.equ	MCBUsed  = 5
.equ	ProcMask = 0x0F
.equ	NonProcMask = 0xF0
.equ	MaxProcNum = 0x0FFF


.equ	MMAP0 = 0x0000
.equ	MMAP1 = 0x4000
.equ	MMAP2 = 0x8000
.equ	MMAP3 = 0xC000


.equ	MaxMemoryLW = 0x0000
.equ	MaxMemoryHW = 0x0002	;max memory 0x00020000 == 128kb

;
;**************************************************************************
;* Hardware Def.
;
;***************************************************************************
;**** VARIABLES
.DSEG


LargeMCB:	.byte	3	;Egybefuggo nagy blokkok >4kb
SmallMCB:	.byte	3	;kicsi blokkok <4kb kezdopozicioja



;***************************************************************************
.ESEG


;***************************************************************************
;**** CODE SEG
;***************************************************************************
.CSEG

ExtSRAMTest:
		ldi	R20,SRAMBASE		;External SRAM Base Address
		ldi	R18,8			;8x16k page = 128k
		ldi	R21,0x5A		;test byte
SRT0:		sts	ADR_EXHH,R20		;Upper memory to SRAM Banks
		ldi	ZL,0
		ldi	ZH,192			;Z point to upper memory
		ldi	XL,low(16384)
		ldi	XH,high(16384)		;Count of test byte

srt1:		st	Z+,R21
		sbiw	XL,1
		brne	srt1			;fill bank
		inc	R20
		dec	R18
		brne	SRT0



		ldi	R20,SRAMBASE
		ldi	R18,8
		ldi	R21,0x5A		;test byte
SRT2:		sts	ADR_EXHH,R20
		ldi	ZL,0
		ldi	ZH,192			;Z point to upper memory
		ldi	XL,low(16384)
		ldi	XH,high(16384)		;Count of test byte

srt3:		ld	R3,Z+
		cp	R21,R3
		brne	SRAMERR
		sbiw	XL,1
		brne	srt3			;check bank
		inc	R20
		dec	R18
		brne	srt2

		clc
		ret

SRAMERR:	sec
		ret

;******************************************************
; Convert Phisycal address to Logical Address
; In:  R18:R17:R16 Phisycal address
; Out  R18 - page,
;      R17:R16 - offset address (must add MMAP1,2,3)
;
MmTranslatePhysicalAddress:
		push	R17
		rol	R17
		rol	R18
		rol	R17
		rol	R18
		pop	R17
		andi	R17,0b00111111
		ret


;******************************************************
; Set Page at R18:R16
; In:  R18:R17:R16 Phisycal address
; Out  R18 - page+1,
;      R17:R16 - offset address on MMAP2
;      Z - mapped address on MMAP2
;
MmSetPage:
		rcall	MmTranslatePhysicalAddress
MmSetPageMap:
		ori	R17,high(MMAP2)
		sts	ADR_EXHL,R18
		inc	R18
		sts	ADR_EXHH,R18		;2 page -> no page border fault
		movw	ZL,R16			;Z - offset at first page
		RET




;******************************************************
; Initialize Memory & setup MCBs
;
Mem_Init:
		ldi	R16,S1D_DISPLAY_HEIGHT
		ldi	R17,S1D_DISPLAY_SCANLINE_BYTES
		mul	R16,R17
		inc	R1			;skip Video RAM + 256 byte
		clr	R2

		sts	LargeMCB+0,R0
		sts	LargeMCB+1,R1
		sts	LargeMCB+2,R2		;First MCB
		movw	R16,R0
		mov	R18,R2
		rcall	MmSetPage

		ldi	R16,(1<<MCBLast) | (1<<MCBFirst)
		call	WaitToDMA
		std	Z+MCB_Type,R16		;First & Last block, not used
		std	Z+MCB_Proc,R2		;Proc = 0, not used
		ldi	R16,low(MaxMemoryLW)
		ldi	R17,high(MaxMemoryLW)
		ldi	R18,low(MaxMemoryHW)	;128kb full memory - MCB_Size
		ldi	R19,MCB_Size
		add	R0,R19
		adc	R1,R2
		adc	R2,R2
		sub	R16,R0
		sbc	R17,R1
		sbc	R18,R2			;R18:R16 Lenght
		std	Z+MCB_LenL+0,R16
		std	Z+MCB_LenL+1,R17
		std	Z+MCB_LenH,R18
		clr	R16
		std	Z+MCB_PrevL+0,R16	;no prev block
		std	Z+MCB_PrevL+1,R16
		std	Z+MCB_PrevH,R16
		ret


;******************************************************
; Dump memory handling 
;

MMLarge:	.db	"Large Blocks:",CR
		.db	"Base   Len    Flag ",CR
		.db	"000000 009700 xxxx Video RAM",CR,0

MmDumpMemoryAreas:
		ldi	R16,Low(MMLarge)
		ldi	R17,High(MMLarge)
		call	SendStrW

		lds	R0,LargeMCB+0
		lds	R1,LargeMCB+1
		lds	R2,LargeMCB+2		;First MCB
		call	WaitToDMA

MmNextMCB:	movw	R16,R0
		mov	R18,R2	
		rcall	MmSetPage

		push	R0
		push	R1
		push	R2
		push	R0
		push	R1
		mov	R0,R2
		call	HEXtoUART
		pop	R0
		call	HEXtoUART
		pop	R0
		call	HEXtoUART
		ldi	R16,32
		call	SendChrW
		ldd	R0,Z+MCB_LenH
		call	HEXtoUART
		ldd	R0,Z+MCB_LenL+1
		call	HEXtoUART
		ldd	R0,Z+MCB_LenL+0
		call	HEXtoUART
		ldi	R16,32
		call	SendChrW
		ldd	R0,Z+MCB_Type
		call	HEXtoUART
		ldd	R0,Z+MCB_Proc
		call	HEXtoUART
		ldi	R16,CR
		call	SendChrW
		pop	R2
		pop	R1
		pop	R0

		ldd	R16,Z+MCB_Type
		sbrc	R16,MCBLast
		 ret

		clr	R3
		ldd	R18,Z+MCB_LenH
		ldd	R17,Z+MCB_LenL+1
		ldd	R16,Z+MCB_LenL+0
		ldi	R19,MCB_Size
		add	R16,R19
		adc	R17,R3
		adc	R18,R3
		add	R0,R16
		adc	R1,R17
		adc	R2,R18
		rjmp	MmNextMCB


;******************************************************
; Allocate memory handling 
; In: R18:R17:R16 - required memory
;     R20:R19 - Process number
;	
;Out: c= 0 no error
;	 R18:R17:16 - phisical Memory pointer (MCB+MCBLen)
;
;     c= 1 error, R17:R16 - error code
;
MmAllocMemory:
		movw	R4,R16
		mov	R6,R18			;R6:R4 - required size

		clr	R15

		cpi	R16,1
		cpc	R17,R15
		cpc	R18,R15
		brsh	mmAlloc01

		ldi	R16,low(ERROR_INVALID_PARAMETER)
		ldi	R17,high(ERROR_INVALID_PARAMETER)
		sec
		ret

mmAlloc01:
		ldi	R21,MCB_Size
		clr	R8
		clr	R9
		clr	R10			;R10:R8 best resut
		
		ldi	R16,0xFF
		mov	R12,R16
		mov	R13,R16
		mov	R14,R16			;min. delta for best result

		lds	R0,LargeMCB+0
		lds	R1,LargeMCB+1
		lds	R2,LargeMCB+2		;R2:R0 MCB pointer

		call	WaitToDMA

MmNextMCB01:	movw	R16,R0
		mov	R18,R2	
		rcall	MmSetPage

		ldd	R16,Z+MCB_Type
		sbrc	R16,MCBUsed
		 rjmp	MMVanMeg

		ldd	R16,Z+MCB_LenL+0		;chunk size
		ldd	R17,Z+MCB_LenL+1
		ldd	R18,Z+MCB_LenH

		sub	R16,R4
		sbc	R17,R5
		sbc	R18,R6
		breq	MMTeljesenJo			;exelent
		brlo	MMVanMeg			;too small
		
		cp	R16,R12
		cpc	R17,R13
		cpc	R18,R14
		brsh	MMVanMeg			;delta>= old delta
		
		movw	R12,R16
		mov	R14,R18		;delta
		movw	R8,R0
		mov	R10,R2		;best pointer
		rjmp	MMVanMeg

MMTeljesenJo:
		movw	R8,R0
		mov	R10,R2		;pointer
		clr	R12
		clr	R13
		clr	R14		;delta=0
		rjmp	MMLastInChain


MMVanMeg:
		ldd	R16,Z+MCB_Type
		sbrc	R16,MCBLast
		 rjmp	MMLastInChain

		ldd	R18,Z+MCB_LenH
		ldd	R17,Z+MCB_LenL+1
		ldd	R16,Z+MCB_LenL+0
		add	R16,R21
		adc	R17,R15
		adc	R18,R15
		add	R0,R16
		adc	R1,R17
		adc	R2,R18
		rjmp	MmNextMCB01

MMLastInChain:
		mov	R0,R8
		or	R0,R9
		or	R0,R10
		brne	mmMoreM00
		rjmp	mmNoMoreMemory
mmMoreM00:
					;Memory block Allocation/sharing
					;Alloc = delta < 40 byte (32+8)
					;share = delta => 40 byte
		ldi	R16,40
		clr	R17
		cp	R12,R16
		cpc	R13,R17
		cpc	R14,R17
		brsh	MMShare

mmAllocAll:	
		movw	R0,R8
		mov	R2,R10
		movw	R16,R8
		mov	R18,R10
		rcall	MmSetPage

		ldd	R16,Z+MCB_Type		;set MCBUsed & Process
		ori	R16,(1<<MCBUsed)
		andi	R16,NonProcMask
		andi	R20,ProcMask
		or	R16,R20
		std	Z+MCB_Type,R16
		std	Z+MCB_Proc,R19		;minden mas valtozatlan marad
		rjmp	MMOk00

MMShare:
		movw	R0,R8
		mov	R2,R10
		movw	R16,R8
		mov	R18,R10
		rcall	MmSetPage		

		ldd	R16,Z+MCB_Type		;set MCBUsed & Process
		push	R16			;save old type
		cbr	R16,(1<<MCBLast)	;clear MCBLast bit
		ori	R16,(1<<MCBUsed)	;set MCBUsed
		andi	R16,NonProcMask
		andi	R20,ProcMask		;set process number
		or	R16,R20
		std	Z+MCB_Type,R16
		std	Z+MCB_Proc,R19		;minden mas valtozatlan marad

		push	R0
		push	R1
		push	R2			;save pointer of this block

		ldd	R11,Z+MCB_LenL+0
		push	R11
		ldd	R11,Z+MCB_LenL+1
		push	R11
		ldd	R11,Z+MCB_LenH
		push	R11

		std	Z+MCB_LenL+0,R4		;block size
		std	Z+MCB_LenL+1,R5
		std	Z+MCB_LenH,R6

		add	R0,R4
		adc	R1,R5
		adc	R2,R6
		add	R0,R21
		adc	R1,R15
		adc	R2,R15			;new blokk pointer

		movw	R16,R0
		mov	R18,R2
		rcall	MmSetPage		;Z - new page 

		pop	R18
		pop	R17
		pop	R16			;old block size
		sub	R16,R21
		sbc	R17,R15
		sbc	R18,R15
		sub	R16,R4
		sbc	R17,R5
		sbc	R18,R6			;R18:R16 new block size

		std	Z+MCB_LenL+0,R16
		std	Z+MCB_LenL+1,R17
		std	Z+MCB_LenH,R18

		pop	R18
		pop	R17
		pop	R16			;prev. block address

		std	Z+MCB_PrevL+0,R16
		std	Z+MCB_PrevL+1,R17
		std	Z+MCB_PrevH,R18

		pop	R16			;old Type
		cbr	R16,(1<<MCBUsed) | (1<<MCBFirst) ;clear MCBUsed & MCBFirst
		andi	R16,NonProcMask
		std	Z+MCB_Type,R16
		std	Z+MCB_Proc,R15		;Process = 0 == none
MMOk00:
		movw	R16,R8
		mov 	R18,R10
		add	R16,R21
		adc	R17,R15
		adc	R18,R15			;MCB+8 real address of free RAM
		clc
		ret

mmNoMoreMemory:
		ldi	R16,low(ERROR_NOT_ENOUGH_MEMORY)
		ldi	R17,high(ERROR_NOT_ENOUGH_MEMORY)
		sec
		ret



;******************************************************
; Release Allocated memory
; In: R18:R17:R16 - Allocated memory (MCB)
;     R20:R19 - Process number
;	
;Out: c= 0 no error
;
;     c= 1 error
;	 R17:R16 - error code
;
MmReleaseMemory:
		clr	R0
		clr	R1
		clr	R2
		clr	R3
		clr	R14
		clr	R15
		ldi	R21,MCB_Size
		movw	R4,R16
		mov	R6,R18			;R6:R4 - MCB pointer
		rcall	MmSetPage
		call	WaitToDMA		;validate MCB


		ldd	R16,Z+MCB_Type
		sbrs	R16,MCBUsed
		 rjmp	mmInvalidHandle		;MCB not used
		andi	R16,ProcMask
		andi	R20,ProcMask
		ldd	R17,Z+MCB_Proc
		cp	R19,R17
		cpc	R20,R16
		breq	mmMakeFree
		rjmp	mmInvalidHandle		;MCB Prox <> Proc

mmMakeFree:
		ldd	R16,Z+MCB_Type
		cbr	R16,(1<<MCBUsed)	;clear MCBUsed bit
		andi	R16,nonProcMask
		mov	R13,R16			;keep last block bit, if exist
		std	Z+MCB_Type,R16
		std	Z+MCB_Proc,R15		;Proc=0=none

		ldd	R7,Z+MCB_PrevL+0	;previous block base 
		ldd	R8,Z+MCB_PrevL+1
		ldd	R9,Z+MCB_PrevH

		ldd	R10,Z+MCB_LenL+0
		ldd	R11,Z+MCB_LenL+1
		ldd	R12,Z+MCB_LenH

		sbrc	R16,MCBLast		;Last block??
		 rjmp	mmRelx00

		movw	R16,R4
		mov	R18,R6
		add	R16,R10
		adc	R17,R11
		adc	R18,R12
		add	R16,R21
		adc	R17,R15
		adc	R18,R15
		rcall	MmSetPage		;next block
		ldd	R16,Z+MCB_Type
		sbrc	R16,MCBUsed		;Used block??
		 rjmp	mmRelx00		;Yes, Used block

		mov	R14,R16			;save type byte
		ldd	R16,Z+MCB_LenL+0
		ldd	R17,Z+MCB_LenL+1
		ldd	R18,Z+MCB_LenH
		add	R0,R16
		adc	R1,R17
		adc	R2,R18			;block length
		add	R0,R21
		adc	R1,R15
		adc	R2,R15

mmRelx00:
		add	R0,R10
		adc	R1,R11
		adc	R2,R12

		sbrc	R13,MCBFirst
		 rjmp	mmRelx01

		mov	R16,R7
		mov	R17,R8
		mov	R18,R9
		rcall	MmSetPage		;prev block
		ldd	R16,Z+MCB_Type
		sbrc	R16,MCBUsed		;Used block??
		 rjmp	mmRelx01		;Yes, Used block

		mov	R3,R16
		ldd	R16,Z+MCB_LenL+0
		ldd	R17,Z+MCB_LenL+1
		ldd	R18,Z+MCB_LenH
		add	R0,R16
		adc	R1,R17
		adc	R2,R18
		add	R0,R21
		adc	R1,R15
		adc	R2,R15

		mov	R4,R7
		mov	R5,R8
		mov	R6,R9			;move block pointer

mmRelx01:
		movw	R16,R4
		mov	R18,R6
		rcall	MmSetPage
	
		std	Z+MCB_LenL+0,R0
		std	Z+MCB_LenL+1,R1
		std	Z+MCB_LenH,R2
		ldi	R16,(1<<MCBLast) | (1<<MCBFirst)
		and	R3,R16
		and	R13,R16
		and	R14,R16
		or	R3,R13
		or	R3,R14
		std	Z+MCB_Type,R3
		std	Z+MCB_Proc,R15		;Proc=0=none

		sbrc	R3,MCBLast
		 rjmp	mmRel05

		movw	R16,R4
		mov	R18,R6
		add	R16,R0
		adc	R17,R1
		adc	R18,R2
		add	R16,R21
		adc	R17,R15
		adc	R18,R15
		rcall	MmSetPage

		std	Z+MCB_PrevL+0,R4
		std	Z+MCB_PrevL+1,R5
		std	Z+MCB_PrevH,R6
mmRel05:
		clc
		ret

mmInvalidHandle:
		ldi	R16,low(ERROR_INVALID_HANDLE)
		ldi	R17,high(ERROR_INVALID_HANDLE)
		sec
		ret
